#  Copyright (c) 2019 | Advancing Technology Systems, LLC
#  See LICENSE for any grants of usage, distribution, or modification


def pawn_pay_logo(prog: str, msg: str):
    from colorama import Fore, Style
    print(Fore.BLUE + "\n          #####")
    print(Fore.BLUE + "       #    #    #           " + Fore.CYAN + "Pawn Payment Solutions")
    print(Fore.BLUE + "     #      #      #         " + Fore.WHITE + prog)
    print(Fore.BLUE + "    #       #       # ")
    print(
        Fore.YELLOW + "  #####     " + Fore.BLUE + "#     " + Fore.YELLOW + "#####      " + Fore.RED + msg
    )
    print(
        Fore.YELLOW
        + " #######    "
        + Fore.BLUE
        + "#    "
        + Fore.YELLOW
        + "#######     "
        + Fore.WHITE
        + "Email us: "
        + Fore.CYAN
        + "support@pawn-pay.com"
    )
    print(Fore.YELLOW + "  #####     " + Fore.BLUE + "#" + Fore.YELLOW + "     #####")
    print(Fore.YELLOW + "          ##### ")
    print(Fore.YELLOW + "         #######")
    print(Fore.YELLOW + "          #####\n\n")
    print(Style.RESET_ALL)
